"""
Federation example showing handlers and signatures.
"""

import asyncio
import logging
from datetime import datetime, timezone
from pathlib import Path

from pyfed.models import APCreate, APNote, APPerson, APLike
from pyfed.security import KeyManager
from pyfed.security.http_signatures import HTTPSignatureVerifier
from pyfed.storage import StorageBackend
from pyfed.federation import ActivityDelivery
from pyfed.handlers import CreateHandler, LikeHandler
from pyfed.protocols.webfinger import WebFingerClient

logger = logging.getLogger(__name__)

async def federation_example():
    """Example of federation with handlers and signatures."""
    try:
        # Initialize components
        storage = StorageBackend.create(
            provider="sqlite",
            database_url="example_data/pyfed_example.db"
        )
        await storage.initialize()

        # Initialize key manager and signature verifier
        key_manager = KeyManager(
            domain="example.com",
            private_key_path="example_keys/private.pem",
            public_key_path="example_keys/public.pem"
        )
        await key_manager.initialize()

        signature_verifier = HTTPSignatureVerifier(key_manager)

        # Initialize handlers
        create_handler = CreateHandler(storage)
        like_handler = LikeHandler(storage)

        # Initialize delivery with signature support
        delivery = ActivityDelivery(
            key_manager=key_manager,
            signature_verifier=signature_verifier
        )

        # Create and send a note
        actor = APPerson(
            id="https://example.com/users/alice",
            name="Alice",
            preferred_username="alice",
            inbox="https://example.com/users/alice/inbox",
            outbox="https://example.com/users/alice/outbox"
        )

        note = APNote(
            id=f"https://example.com/notes/{datetime.now(timezone.utc).timestamp()}",
            content="Hello Fediverse!",
            attributed_to=str(actor.id),
            to=["https://www.w3.org/ns/activitystreams#Public"]
        )

        # Create activity
        create_activity = APCreate(
            id=f"https://example.com/activities/{datetime.now(timezone.utc).timestamp()}",
            actor=str(actor.id),
            object=note,
            to=note.to
        )

        # Process outgoing activity through handler
        await create_handler.handle_outgoing(create_activity.serialize())

        # Sign and deliver activity
        activity_dict = create_activity.serialize()
        signed_headers = await signature_verifier.sign_request(
            method="POST",
            path="/inbox",
            body=activity_dict
        )

        # Deliver with signed headers
        result = await delivery.deliver_activity(
            activity=activity_dict,
            recipients=["https://remote.example/inbox"],
            headers=signed_headers
        )

        # Simulate receiving an activity
        incoming_headers = {
            "Signature": "...",  # Signature from remote server
            "Date": "...",
            "Host": "example.com"
        }
        incoming_activity = {
            "type": "Like",
            "actor": "https://remote.example/users/bob",
            "object": note.id
        }

        # Verify signature of incoming activity
        if await signature_verifier.verify_request(incoming_headers):
            # Process through handler
            await like_handler.handle_incoming(incoming_activity)
        else:
            logger.error("Invalid signature on incoming activity")

    finally:
        await storage.close()

if __name__ == "__main__":
    asyncio.run(federation_example()) 