"""
Note creation and interaction examples.
"""

import asyncio
import logging
from datetime import datetime, timezone
import os
import sys
from pathlib import Path
import ssl
from urllib.parse import urlparse

# Add src directory to Python path
src_path = Path(__file__).parent.parent / "src"
sys.path.insert(0, str(src_path))

from pyfed.models import APCreate, APNote, APPerson
from pyfed.security import KeyManager
from pyfed.storage import StorageBackend
from pyfed.federation import ActivityDelivery
from pyfed.protocols.webfinger import WebFingerClient
from pyfed.security.http_signatures import HTTPSignatureVerifier
from pyfed.serializers.json_serializer import to_json, from_json

# Configure logging
logging.basicConfig(level=logging.INFO)
logger = logging.getLogger(__name__)

def ensure_directories():
    """Ensure required directories exist."""
    dirs = [
        "example_keys",
        "example_data",
        "example_media"
    ]
    
    for dir_name in dirs:
        path = Path(dir_name)
        path.mkdir(parents=True, exist_ok=True)
        logger.info(f"Ensured directory exists: {path}")

async def create_note_example():
    """Example of creating and delivering a note."""
    try:
        ensure_directories()

        # Initialize storage
        storage = StorageBackend.create(
            provider="sqlite",
            database_url="example_data/pyfed_example.db"
        )
        await storage.initialize()

        # Initialize key manager
        key_manager = KeyManager(
            domain="example.com",
            active_keys=Path("example_keys").resolve(),
            keys_path=str(Path("example_keys").resolve())
        )
        await key_manager.initialize()

        # Get the active key for signing
        active_key = await key_manager.get_active_key()
        logger.info(f"Active key: {active_key}")

        # Initialize HTTP signature verifier with the active key paths
        logger.info(f"Using key paths: {key_manager.keys_path}/{active_key.key_id}_private.pem and {key_manager.keys_path}/{active_key.key_id}_public.pem")
        signature_verifier = HTTPSignatureVerifier(
            private_key_path=f"{key_manager.keys_path}/{active_key.key_id}_private.pem",
            public_key_path=f"{key_manager.keys_path}/{active_key.key_id}_public.pem"
        )

        # Initialize delivery
        delivery = ActivityDelivery(
            key_manager=key_manager,
            timeout=30
        )
        await delivery.initialize()

        # Initialize WebFinger client with SSL verification disabled for testing
        webfinger = WebFingerClient(verify_ssl=False)
        await webfinger.initialize()

        # Get inbox URL for recipient
        recipient = "kene29@mastodon.social"
        logger.info(f"Looking up inbox for {recipient}...")
        inbox_url = await webfinger.get_inbox_url(recipient)
        if not inbox_url:
            logger.error(f"Could not find inbox for {recipient}")
            return

        logger.info(f"Found inbox URL: {inbox_url}")

        # Create actor
        actor = APPerson(
            id="https://example.com/users/alice",
            name="Alice",
            preferred_username="alice",
            inbox="https://example.com/users/alice/inbox",
            outbox="https://example.com/users/alice/outbox",
            followers="https://example.com/users/alice/followers"
        )

        # Create note with string attributed_to
        note = APNote(
            id=f"https://example.com/notes/{datetime.now(timezone.utc).timestamp()}",
            content=f"Hello @{recipient}! This is a test note!",
            attributed_to=str(actor.id),  # Convert URL to string
            to=[inbox_url],
            cc=["https://www.w3.org/ns/activitystreams#Public"],
            published=datetime.now(timezone.utc).isoformat()
        )

        # Create activity
        create_activity = APCreate(
            id=f"https://example.com/activities/{datetime.now(timezone.utc).timestamp()}",
            actor=str(actor.id),  # Convert URL to string
            object=note,
            to=note.to,
            cc=note.cc,
            published=datetime.now(timezone.utc).isoformat(),
        )

        # Serialize and deliver
        logger.info("Serializing activity...")
        activity_dict = create_activity.serialize()
        # logger.info(f"Serialized activity: {activity_dict}")
        # logger.info(f"Activity: {to_json(create_activity, indent=2)}")

        # Sign the request
        parsed_url = urlparse(inbox_url)
        headers = {
            "Accept": "application/ld+json; profile=\"https://www.w3.org/ns/activitystreams\"",
            "Content-Type": "application/ld+json; profile=\"https://www.w3.org/ns/activitystreams\"",
            "User-Agent": "PyFed/1.0",
            "Host": parsed_url.netloc
        }
        logger.debug(f"Resolved host: {urlparse(inbox_url).netloc}")
        signed_headers = await signature_verifier.sign_request(
            method="POST",
            path="/inbox",
            headers=headers,
            body=activity_dict
        )

        logger.debug(f"Headers before signing: {signed_headers}")

        logger.info("Delivering activity...")
        result = await delivery.deliver_activity(
            activity=activity_dict,
            recipients=[inbox_url],
            headers=signed_headers  # Include signed headers
        )
        logger.info(f"Delivery result: {result}")

    except Exception as e:
        logger.error(f"Error in note example: {e}")
        raise
    finally:
        if 'storage' in locals():
            await storage.close()
        if 'delivery' in locals():
            await delivery.close()
        if 'webfinger' in locals():
            await webfinger.close()

def main():
    """Main entry point."""
    try:
        asyncio.run(create_note_example())
    except KeyboardInterrupt:
        logger.info("Example stopped by user")
    except Exception as e:
        logger.error(f"Example failed: {e}")
        sys.exit(1)

if __name__ == "__main__":
    main() 