"""
PyFed runnable example.
"""

import asyncio
import logging
from datetime import datetime
import os
import sys
from pathlib import Path

# Add src directory to Python path
src_path = Path(__file__).parent.parent / "src"
sys.path.insert(0, str(src_path))

from pyfed.models import APCreate, APNote, APPerson, APLike
from pyfed.security import KeyManager
from pyfed.storage import StorageBackend
from pyfed.federation import ActivityDelivery
from pyfed.serializers.json_serializer import to_json, from_json

# Configure logging
logging.basicConfig(level=logging.INFO)
logger = logging.getLogger(__name__)

def ensure_directories():
    """Ensure required directories exist."""
    dirs = [
        "example_keys",
        "example_data",
        "example_media"
    ]
    
    for dir_name in dirs:
        path = Path(dir_name)
        path.mkdir(parents=True, exist_ok=True)
        logger.info(f"Ensured directory exists: {path}")

async def run_examples():
    """Run PyFed examples."""
    try:
        logger.info("Setting up PyFed components...")
        ensure_directories()

        # Initialize storage
        storage = StorageBackend.create(
            provider="sqlite",
            database_url="example_data/pyfed_example.db"
        )
        await storage.initialize()

        # Initialize key manager
        key_manager = KeyManager(
            domain="example.com",
            active_keys=Path("example_keys").resolve(),
            keys_path=str(Path("example_keys").resolve())
        )
        await key_manager.initialize()

        # Initialize delivery
        delivery = ActivityDelivery(
            key_manager=key_manager,
            timeout=30
        )

        # Create actor
        actor = APPerson(
            id="https://example.com/users/alice",
            name="Alice",
            preferred_username="alice",
            inbox="https://example.com/users/alice/inbox",
            outbox="https://example.com/users/alice/outbox",
            followers="https://example.com/users/alice/followers"
        )

        # Create note
        note = APNote(
            id="https://example.com/notes/123",
            content="Hello, Federation! #test @bob@remote.com",
            attributed_to=str(actor.id),
            to=["https://www.w3.org/ns/activitystreams#Public"],
            published=datetime.utcnow().isoformat()
        )

        # Create activity
        create_activity = APCreate(
            id=f"https://example.com/activities/{datetime.utcnow().timestamp()}",
            actor=str(actor.id),
            object=note,
            to=note.to,
            published=datetime.utcnow().isoformat()
        )

        # Store activity
        logger.info("Storing activity...")
        activity_dict = create_activity.serialize()
        logger.info(f"Serialized activity (with context):\n{to_json(create_activity, indent=2)}")
        activity_id = await storage.create_activity(activity_dict)
        logger.info(f"Activity stored with ID: {activity_id}")

        # Create like activity
        like_activity = APLike(
            id=f"https://example.com/activities/like_{datetime.utcnow().timestamp()}",
            actor=str(actor.id),
            object=note.id,
            to=["https://www.w3.org/ns/activitystreams#Public"],
            published=datetime.utcnow().isoformat()
        )

        # Store like
        logger.info("Storing like activity...")
        like_dict = like_activity.serialize()
        logger.info(f"Serialized like activity (with context):\n{to_json(like_activity, indent=2)}")
        like_id = await storage.create_activity(like_dict)
        logger.info(f"Like activity stored with ID: {like_id}")

        # Retrieve activities
        logger.info("\nRetrieving activities...")
        stored_activity = await storage.get_activity(activity_id)
        stored_like = await storage.get_activity(like_id)

        logger.info("Retrieved Create activity:")
        logger.info(stored_activity)
        logger.info("\nRetrieved Like activity:")
        logger.info(stored_like)

    except Exception as e:
        logger.error(f"Error running examples: {e}")
        raise
    finally:
        if storage:
            await storage.close()

def main():
    """Main entry point."""
    try:
        asyncio.run(run_examples())
    except KeyboardInterrupt:
        logger.info("Example stopped by user")
    except Exception as e:
        logger.error(f"Example failed: {e}")
        sys.exit(1)

if __name__ == "__main__":
    main() 